/*
  Copyright 1996,1997 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// This is the main CGI program which extracts DAS class attributes from a
// matlab file. In addition, the attributes from an external file (if 
// exist, with the same name as the matlab file and extension of .das at the 
// end) are extracted and added to the in-memory atrribute table before it is 
// sent over the network to the client side. 
//
// ReZa 9/28/96
//

// $Log: mat_das.cc,v $
// Revision 1.8  1999/05/04 03:30:52  jimg
// Merged no gnu changes
//
// Revision 1.7.8.1  1999/04/09 05:29:01  brent
// convert String.h code to new standard lib <string>
//
// Revision 1.7  1998/03/20 00:32:21  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.6  1997/12/30 23:56:09  jimg
// Fixed errant comments.
//
// Revision 1.5  1997/12/30 22:45:16  jimg
// Changed from old style filter to an implementation that uses the DODSFilter
// class.
//
// Revision 1.4  1997/10/04 00:33:21  jimg
// Release 2.14c fixes
//
// Revision 1.3.6.1  1997/09/03 22:21:20  jimg
// Changed to work with the modified MatLabUtil.cc file; global objects are
// now pointers.
//
// Revision 1.3  1997/06/06 03:57:21  jimg
// Changed prototype and call to read_descriptors().
//
// Revision 1.2  1997/05/01 18:36:04  jimg
// Added configureation header.
// Merged changes from interim 2.1.2 version onto main trunk.
//
// Revision 1.1  1996/10/31 14:43:41  reza
// First release of DODS-matlab servers.

static char rcsid[]={"$Id: mat_das.cc,v 1.8 1999/05/04 03:30:52 jimg Exp $"};

#include <iostream.h>
#include <string>

#include "config_mat.h"

#include "DAS.h"
#include "cgi_util.h"
#include "DODSFilter.h"

extern bool read_attributes(DAS &das, string filename, string *error);

int 
main(int argc, char *argv[])
{
    DAS das;
    DODSFilter df(argc, argv);

    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

    // Read the attributes in memory and forward all the errors to client
    // program
    string errMsg;
    if(!read_attributes(das, df.get_dataset_name(), &errMsg)){
      Error e(unknown_error, errMsg);
      set_mime_text(cout,dods_error);
      e.print(cout);
      return 1;
    }

    if (!df.read_ancillary_das(das))
	return 1;

    if (!df.send_das(das))
	return 1;

    return 0;
}
